class settings:
    STRICT = False 

from .encoding import *

from .ascii85 import *
from .fontmetrics import FONT_METRICS
from .generic import * 
from .lzw import * 
from .matrix import *
from .text import *
from .types import *
from .log import *